import { ProjectPrincipal } from './ProjectPrincipal.js';
import { join, toAbsolute } from './util/path.js';
const mergePaths = (cwd, compilerOptions, paths = {}) => {
    const overridePaths = Object.keys(paths).reduce((overridePaths, key) => {
        overridePaths[key] = paths[key].map(entry => toAbsolute(entry, cwd));
        return overridePaths;
    }, {});
    compilerOptions.paths = { ...compilerOptions.paths, ...overridePaths };
    return compilerOptions;
};
export class PrincipalFactory {
    principals = new Set();
    getPrincipal({ cwd, compilerOptions, paths, compilers }) {
        compilerOptions = mergePaths(cwd, compilerOptions, paths);
        const principal = this.findReusablePrincipal(compilerOptions);
        if (principal) {
            this.linkPrincipal(principal, cwd, compilerOptions.paths);
            return principal.principal;
        }
        else {
            return this.addNewPrincipal({ cwd, compilerOptions, paths, compilers });
        }
    }
    findReusablePrincipal(compilerOptions) {
        const workspacePaths = compilerOptions?.paths ? Object.keys(compilerOptions.paths) : [];
        const principal = Array.from(this.principals).find(principal => {
            if (compilerOptions.baseUrl === principal.principal.compilerOptions.baseUrl) {
                return workspacePaths.every(p => !principal.pathKeys.has(p));
            }
            return !compilerOptions.baseUrl;
        });
        return principal;
    }
    linkPrincipal(principal, cwd, paths) {
        Object.keys(paths ?? {}).forEach(p => principal.pathKeys.add(p));
        principal.principal.compilerOptions.paths = { ...principal.principal.compilerOptions.paths, ...paths };
        principal.cwds.add(cwd);
    }
    addNewPrincipal({ cwd, compilerOptions, compilers }) {
        const principal = new ProjectPrincipal({ cwd, compilerOptions, compilers });
        const pathKeys = new Set(Object.keys(compilerOptions?.paths ?? {}));
        compilerOptions.baseUrl = join(cwd, compilerOptions.baseUrl ?? '.');
        this.principals.add({ principal, cwds: new Set([cwd]), pathKeys });
        return principal;
    }
    getPrincipals() {
        return Array.from(this.principals).map(p => p.principal);
    }
}
