import { compact } from '../../util/array.js';
import { timerify } from '../../util/Performance.js';
import { hasDependency, load } from '../../util/plugin.js';
import { getEnvPackageName, getExternalReporters } from './helpers.js';
export const NAME = 'Vitest';
export const ENABLERS = ['vitest'];
export const isEnabled = ({ dependencies }) => hasDependency(dependencies, ENABLERS);
export const ENTRY_FILE_PATTERNS = ['vite.config.ts'];
export const CONFIG_FILE_PATTERNS = ['vitest.config.ts'];
const findVitestDependencies = async (configFilePath) => {
    const config = await load(configFilePath);
    if (!config || !config.test)
        return [];
    const cfg = config.test;
    const environments = cfg.environment ? [getEnvPackageName(cfg.environment)] : [];
    const reporters = getExternalReporters(cfg.reporters);
    const coverage = cfg.coverage?.provider ? [`@vitest/coverage-${cfg.coverage.provider}`] : [];
    return compact([...environments, ...reporters, ...coverage]);
};
export const findDependencies = timerify(findVitestDependencies);
